#include <stdio.h>
#include <math.h>
#include <time.h>

typedef long CPU_clock_t;

CPU_clock_t 
RPCC(void)
{
#if defined(__GNUC__) && defined(__alpha__)
  static long	clock_offset = 0L;
  static long	t_last = 0L;
  long		t;
  long		u;
  asm("subq %0,%0,%0 ; rpcc %0; sll  %0, 32, %1; addq %0, %1, %0; srl  %0, 32, %0" : "=r" (t) : "r" (u));

  return (t);
#else
  return (0L);
#endif
}

CPU_clock_t 
CPU_clock(void)
{
#if defined(__GNUC__) && defined(__alpha__)
  static long	clock_offset = 0L;
  static long	t_last = 0L;
  long		t;
  long		u;
#if 0
	rpcc	$0		# read the process cycle counter 
	sll	$0, 32, $1	# line up the offset and count fields
	addq	$0, $1, $0	# do add
	srl	$0, 32, $0	# zero extend the cycle count to 64 bits
#endif
  asm("rpcc %0; sll  %0, 32, %1; addq %0, %1, %0; srl  %0, 32, %0" : "=r" (t) : "r" (u));

  if (t < t_last)			/* then 32-bit cycle timer wrapped */
	clock_offset += 0x100000000L;
  t_last = t;
  return (clock_offset + t);
#else
  return (1000000L + sizeof(CPU_clock_t));
#endif
}




int main(void)
{
  CPU_clock_t c;
  CPU_clock_t c_begin,c_end;
  CPU_clock_t r;
  CPU_clock_t r_begin,r_end;
  clock_t t,t_begin,t_end;
  long k;

  t_begin = clock();
  c_begin = CPU_clock();
  r_begin = RPCC();

  printf("%16ld\t%016lx\t%lu\t%016lx\t%lu\n", (long)0, 
	 (long)c_begin, (unsigned long)c_begin, 
	 (long)r_begin, (unsigned long)r_begin);

  t_begin = clock();
  c_begin = CPU_clock();
  r_begin = RPCC();

  printf("%16ld\t%016lx\t%lu\t%016lx\t%lu\n", (long)0, 
	 (long)c_begin, (unsigned long)c_begin, 
	 (long)r_begin, (unsigned long)r_begin);

  for (k = 1; k <= 50000000L; ++k)
  {
	(void)sin(3.12345);
	c = CPU_clock();
	r_end = r = RPCC();
	if ((k % 50000L) == 0)
	{
	    printf("%16ld\t%016lx\t%lu\t%016lx\t%lu\n", 
		   (long)k, (long)c, (unsigned long)c, (long)r, (unsigned long)r);
	}
	if ((k % 500000L) == 0)
	{
#if 1
	    printf("%16ld\t%016lx\t%lu\t%016lx\t%lu\n", 
		   (long)k, (long)c, (unsigned long)c, (long)r, (unsigned long)r);
#endif
	    t_end = t = clock();
	    if ((t - t_begin) > 5 * CLOCKS_PER_SEC) /* bail out after 2 sec */
		break;
	}
  }

  c_end = CPU_clock();

  printf("Time for test:      %10.3f sec\n", 
	 ((double)(t_end - t_begin)/(double)CLOCKS_PER_SEC));
  
  printf("CPU cycle range:    %10ld -- %10ld: delta = %10ld\n", 
	 (long)c_begin, (long)c_end, (long)(c_end - c_begin));

  printf("RPCC range:         %10ld -- %10ld: delta = %10ld\n", 
	 (long)r_begin, (long)r_end, (long)(r_end - r_begin));

  printf("clock range:        %10ld -- %10ld: delta = %10ld\n", 
	 (long)t_begin, (long)t_end, (long)(t_end - t_begin));

  printf("CLOCKS_PER_SEC:     %10ld\n", CLOCKS_PER_SEC);

  printf("Apparent CPU speed: %10.3f MHz\n",
	 ((double)(c_end - c_begin)/((double)(t_end - t_begin)/(double)CLOCKS_PER_SEC))/1000000.0);

  return (0);
}
