*** 20020210/lib.c	Thu Aug 31 09:17:22 2000
--- ./lib.c	Sat Mar 16 08:58:07 2002
***************
*** 475,480 ****
--- 475,487 ----
  
  int	errorflag	= 0;
  
+ #if defined(__cplusplus)
+ void yyerror(const char *s)
+ {
+ 	SYNTAX((char*)s);
+ }
+ #endif
+ 
  void yyerror(char *s)
  {
  	SYNTAX(s);
*** 20020210/main.c	Fri Mar  1 11:40:58 2002
--- ./main.c	Sat Mar 16 08:08:38 2002
***************
*** 22,28 ****
--- 22,33 ----
  THIS SOFTWARE.
  ****************************************************************/
  
+ #if defined(PROFILE)
+ char	*version = "version 20020210, with profiling and global-variable enhancements 1996.10.07 and 2002.03.16";
+ #else
  char	*version = "version 20020210";
+ #endif
+ 
  
  #define DEBUG
  #include <stdio.h>
***************
*** 50,58 ****
--- 55,78 ----
  
  int	safe	= 0;	/* 1 => "safe" mode */
  
+ #if defined(PROFILE)
+ int	g_argc;
+ char	**g_argv;
+ #endif
+ 
  int main(int argc, char *argv[])
  {
  	char *fs = NULL;
+ 
+ #if defined(PROFILE)
+ 	/* We need to preserve original arguments for use in profile
+ 	   table, because argv[] gets modified below. */
+ 	g_argc = argc;
+ 	g_argv = (char**)malloc((argc + 1) * sizeof(g_argv[0]));
+ 	if (g_argv == (char**)NULL)
+ 	    g_argv = &argv[0];		/* silent recovery from unlikely error */
+ 	(void)memcpy(g_argv,argv,(argc + 1) * sizeof(g_argv[0]));
+ #endif
  
  	cmdname = argv[0];
  	if (argc == 1) {

*** ../nawk-20020210/run.c	Tue Jan  1 07:39:19 2002
--- ./run.c	Sat Mar 16 12:57:28 2002
***************
*** 88,93 ****
--- 88,100 ----
  
  Node	*curnode = NULL;	/* the node being executed, for debugging */
  
+ #if defined(PROFILE)
+ #include "profile.h"
+ 
+ extern int g_argc;
+ extern char **g_argv;
+ #endif /* defined(PROFILE) */
+ 
  /* buffer memory management */
  int adjbuf(char **pbuf, int *psiz, int minlen, int quantum, char **pbptr,
  	char *whatrtn)
***************
*** 140,145 ****
--- 147,155 ----
  	if (u == NULL)
  		return(True);
  	for (a = u; ; a = a->nnext) {
+ #if defined(PROFILE)
+ 		profile_begin_line(a->lineno);
+ #endif
  		curnode = a;
  		if (isvalue(a)) {
  			x = (Cell *) (a->narg[0]);
***************
*** 147,152 ****
--- 157,167 ----
  				fldbld();
  			else if (isrec(x) && !donerec)
  				recbld();
+ 
+ #if defined(PROFILE)
+ 			profile_end_line(a->lineno);
+ #endif
+ 
  			return(x);
  		}
  		if (notlegal(a->nobj))	/* probably a Cell* but too risky to print */
***************
*** 158,169 ****
--- 173,210 ----
  		else if (isrec(x) && !donerec)
  			recbld();
  		if (isexpr(a))
+ 		{
+ 
+ #if defined(PROFILE)
+ 			profile_end_line(a->lineno);
+ #endif
+ 
  			return(x);
+ 		}
  		if (isjump(x))
+ 		{
+ #if defined(PROFILE)
+ 			profile_end_line(a->lineno);
+ #endif
+ 
  			return(x);
+ 
+ 		}
  		if (a->nnext == NULL)
+ 		{
+ 
+ #if defined(PROFILE)
+ 			profile_end_line(a->lineno);
+ #endif
+ 
  			return(x);
+ 		}
  		tempfree(x);
+ 
+ #if defined(PROFILE)
+ 		profile_end_line(a->lineno);
+ #endif
+ 
  	}
  }
  
***************
*** 172,177 ****
--- 213,222 ----
  {				/* a[0] = BEGIN, a[1] = body, a[2] = END */
  	Cell *x;
  
+ #if defined(PROFILE)
+ 	profile_init(g_argc, g_argv);
+ #endif
+ 
  	if (setjmp(env) != 0)
  		goto ex;
  	if (a[0]) {		/* BEGIN */
***************
*** 199,204 ****
--- 244,255 ----
  		tempfree(x);
  	}
    ex1:
+ 
+ #if defined(PROFILE)
+ 	profile();
+ 	profile_term();
+ #endif
+ 
  	return(True);
  }
  

*** 20020210/pawk.1	Sun Aug 24 06:37:08 1997
--- ./pawk.1	Sat Mar 16 11:18:29 2002
***************
*** 8,19 ****
  .ft 1
  ..
  awk
! .TH AWK 1
  .CT 1 files prog_other
  .SH NAME
! awk \- pattern-directed scanning and processing language
  .SH SYNOPSIS
! .B awk
  [
  .BI \-F
  .I fs
--- 8,22 ----
  .ft 1
  ..
  awk
! .TH PAWK 1
  .CT 1 files prog_other
  .SH NAME
! pawk \- pattern-directed scanning and processing language, with profiling support
  .SH SYNOPSIS
! [
! env PROFILE=...
! ]
! .B pawk
  [
  .BI \-F
  .I fs
***************
*** 32,38 ****
  .I file ...
  ]
  .SH DESCRIPTION
! .I Awk
  scans each input
  .I file
  for lines that match any of a set of patterns specified literally in
--- 35,41 ----
  .I file ...
  ]
  .SH DESCRIPTION
! .I Pawk
  scans each input
  .I file
  for lines that match any of a set of patterns specified literally in
***************
*** 49,55 ****
  Each line is matched against the
  pattern portion of every pattern-action statement;
  the associated action is performed for each matched pattern.
! The file name 
  .B \-
  means the standard input.
  Any
--- 52,58 ----
  Each line is matched against the
  pattern portion of every pattern-action statement;
  the associated action is performed for each matched pattern.
! The file name
  .B \-
  means the standard input.
  Any
***************
*** 91,97 ****
  .IP
  .IB pattern " { " action " }
  .PP
! A missing 
  .BI { " action " }
  means print the line;
  a missing pattern always matches.
--- 94,100 ----
  .IP
  .IB pattern " { " action " }
  .PP
! A missing
  .BI { " action " }
  means print the line;
  a missing pattern always matches.
***************
*** 195,201 ****
  .BR sin ,
  .BR cos ,
  and
! .BR atan2 
  are built in.
  Other built-in functions:
  .TF length
--- 198,204 ----
  .BR sin ,
  .BR cos ,
  and
! .BR atan2
  are built in.
  Other built-in functions:
  .TF length
***************
*** 224,230 ****
  substring of
  .I s
  that begins at position
! .IR m 
  counted from 1.
  .TP
  .BI index( s , " t" )
--- 227,233 ----
  substring of
  .I s
  that begins at position
! .IR m
  counted from 1.
  .TP
  .BI index( s , " t" )
***************
*** 352,358 ****
  of regular expressions and
  relational expressions.
  Regular expressions are as in
! .IR egrep ; 
  see
  .IR grep (1).
  Isolated regular expressions
--- 355,361 ----
  of regular expressions and
  relational expressions.
  Regular expressions are as in
! .IR egrep ;
  see
  .IR grep (1).
  Isolated regular expressions
***************
*** 468,473 ****
--- 471,491 ----
  Parameters are local to the function; all other variables are global.
  Thus local variables may be created by providing excess parameters in
  the function definition.
+ .SH "PROFILING USER AWK PROGRAMS"
+ .B pawk
+ always accumulates profiling data, at some modest cost compared to
+ .BR awk ,
+ but does not write a profile report unless the environment variable
+ .B PROFILE
+ is defined.  Its value determines the destination of the profile report:
+ 1 for
+ .IR stdout ,
+ 2 for
+ .IR stderr ,
+ or a filename, for output to that file.  Should the open of that file
+ fail, then the report is written to
+ .I stderr
+ without complaint.
  .SH EXAMPLES
  .TP
  .EX
***************
*** 512,518 ****
  .fi
  .EE
  .SH SEE ALSO
! .IR lex (1), 
  .IR sed (1)
  .br
  A. V. Aho, B. W. Kernighan, P. J. Weinberger,
--- 530,536 ----
  .fi
  .EE
  .SH SEE ALSO
! .IR lex (1),
  .IR sed (1)
  .br
  A. V. Aho, B. W. Kernighan, P. J. Weinberger,
